"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class Status(str, Enum):
    r"""The status of the webhook."""

    ENABLED = "enabled"
    DISABLED = "disabled"
