"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DrivesFilterTypedDict(TypedDict):
    group_id: NotRequired[str]
    r"""ID of the drive group to filter on"""


class DrivesFilter(BaseModel):
    group_id: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""ID of the drive group to filter on"""
