"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import importlib.metadata

__title__: str = "apideck-unify"
__version__: str = "0.11.1"
__openapi_doc_version__: str = "10.15.2"
__gen_version__: str = "2.568.2"
__user_agent__: str = "speakeasy-sdk/python 0.11.1 2.568.2 10.15.2 apideck-unify"

try:
    if __package__ is not None:
        __version__ = importlib.metadata.version(__package__)
except importlib.metadata.PackageNotFoundError:
    pass
