"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class RequestCountAllocationTypedDict(TypedDict):
    unify: NotRequired[float]
    proxy: NotRequired[float]
    vault: NotRequired[float]


class RequestCountAllocation(BaseModel):
    unify: Optional[float] = None

    proxy: Optional[float] = None

    vault: Optional[float] = None
