"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentFrequency(str, Enum):
    r"""Frequency of employee compensation."""

    WEEKLY = "weekly"
    BIWEEKLY = "biweekly"
    MONTHLY = "monthly"
    PRO_RATA = "pro-rata"
    OTHER = "other"
