"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class SupportedPropertyTypedDict(TypedDict):
    unified_property: NotRequired[str]
    r"""Name of the property in our Unified API."""
    child_properties: NotRequired[List[SupportedPropertyTypedDict]]
    r"""List of child properties of the unified property."""


class SupportedProperty(BaseModel):
    unified_property: Optional[str] = None
    r"""Name of the property in our Unified API."""

    child_properties: Optional[List[SupportedProperty]] = None
    r"""List of child properties of the unified property."""
