"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ProfitAndLossType(str, Enum):
    r"""The type of profit and loss"""

    SECTION = "Section"
    RECORD = "Record"
