"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SchemaSupportTypedDict(TypedDict):
    r"""When a connector has schema_support, a call can be made to retrieve a json schema that describes a downstream resource."""

    supported: NotRequired[bool]
    r"""Can a resource schema be retrieved for this connector?"""


class SchemaSupport(BaseModel):
    r"""When a connector has schema_support, a call can be made to retrieve a json schema that describes a downstream resource."""

    supported: Optional[bool] = None
    r"""Can a resource schema be retrieved for this connector?"""
